/*
 * File Name: 		ImageEngine.java
 * 
 * Copyright(c) 2011 Yunmai Co.,Ltd.
 * 
 * 		 All rights reserved.
 * 					
 */

package com.yunmai.android.engine;

import java.io.UnsupportedEncodingException;

import android.util.Log;

import com.ym.ocr.img.NativeImage;

/**
 * @author Иεtwμ
 * 
 */
public class ImageEngine {

	// format
	public static final int IMG_FMT_UNK = 0;
	public static final int IMG_FMT_BMP = 1;
	public static final int IMG_FMT_JPG = 2;

	// component
	public static final int IMG_COMPONENT_GRAY = 1;
	public static final int IMG_COMPONENT_RGB = 3;

	public static final int RET_OK = 1;
	public static final int RET_ERR_UNK = 0; /* unknown error */
	public static final int RET_ERR_PTR = -1; /* invalid pointer */
	public static final int RET_ERR_ARG = -2; /* error parameter */
	public static final int RET_ERR_MEM = -3; /* fail to allocate memory */

	protected long mEngine = 0;
	protected NativeImage mNativeImage = null;

	// ///////////////////////////////////////////////////////////////////////
	// Constructor/Destructor

	public ImageEngine() {
		mNativeImage = new NativeImage();
		mEngine = mNativeImage.createEngine();
	}

	@Override
	public void finalize() {
		if (mNativeImage != null && mEngine != 0) {
			mNativeImage.freeImage(mEngine);
			mNativeImage.closeEngine(mEngine);
			mEngine = 0;
		}
	}

	// ///////////////////////////////////////////////////////////
	// Methods

	public boolean init(int components, int quality) {
		if (mNativeImage != null) {
			int ret = mNativeImage.initImage(mEngine, components, quality);
			if (ret == RET_OK) {
				return true;
			}
		}
		return false;
	}

	public boolean load(byte[] imgbuffer) {
		if (mNativeImage != null) {
			int ret = mNativeImage.loadmemjpg(mEngine, imgbuffer, imgbuffer.length);
			if (ret == RET_OK) {
				return true;
			}
		}
		return false;
	}

	public boolean load(String path) {
		StringBuffer s = new StringBuffer(path);
		s.append('0');
		byte[] buffer = null;
		try {
			buffer = s.toString().getBytes("UTF-8");
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		buffer[buffer.length - 1] = 0;
		if (mNativeImage != null) {
			int ret = mNativeImage.loadImage(mEngine, buffer, 0);
			if (ret == RET_OK) {
				return true;
			}
			Log.d("tag", ret + "<path>" + path);
		}

		return false;
	}

	public long getDataEx() {
		if (mNativeImage != null) {
			return mNativeImage.getImageDataEx(mEngine);
		}
		return 0;
	}

	public int getWidth() {
		if (mNativeImage != null) {
			return mNativeImage.getImageWidth(mEngine);
		}
		return 0;
	}

	public int getHeight() {
		if (mNativeImage != null) {
			return mNativeImage.getImageHeight(mEngine);
		}
		return 0;
	}

	public int getComponent() {
		if (mNativeImage != null) {
			return mNativeImage.getImageComponent(mEngine);
		}
		return 0;
	}

}
