/*
 * File Name: 		OcrEngine.java
 * 
 * Copyright(c) 2011 Yunmai Co.,Ltd.
 * 
 * 		 All rights reserved.
 * 					
 */

package com.yunmai.android.engine;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import android.content.Context;
import android.content.res.AssetManager;
import android.graphics.Rect;
import android.telephony.TelephonyManager;
import android.util.Log;

import com.ym.idcard.reg.NativeOcr;
import com.yunmai.android.other.FileUtil;
import com.yunmai.android.other.LicenseManager;
import com.yunmai.android.other.StringUtil;
import com.yunmai.android.vo.IDCard;
import com.yunmai.cc.idcard.controler.YMLogUtil;

/**
 * @author Иεtwμ
 * 
 */
public class OcrEngine {

	//
	// OCR language
	private static final int OCR_LAN_NIL = 0x00; // NIL
	private static final int OCR_LAN_ENGLISH = 0x01; // English
	private static final int OCR_LAN_CHINESE_SIMPLE = 0x02; // Chinese Simple
	private static final int OCR_LAN_CHINESE_TRADITIONAL = 0x15; // Chinese
																	// Traditional
	private static final int OCR_LAN_EUROPEAN = 0x03; // English + European
	private static final int OCR_LAN_RUSSIAN = 0x04; // Russian
	private static final int OCR_LAN_JAPAN = 0x06; // Japan
	private static final int OCR_LAN_CENTEURO = 0x07; // Central European

	// KEY language
	private static final int OCR_CODE_NIL = 0;
	private static final int OCR_CODE_GB = 1;
	private static final int OCR_CODE_B5 = 2;
	private static final int OCR_CODE_GB2B5 = 3;

	//
	// return value
	public static final int RECOG_FAIL = -2; // 识别失败
	public static final int RECOG_CANCEL = -1; // 取消识别
	public static final int RECOG_NONE = 0; // do nothing
	public static final int RECOG_OK = 1; // 识别成功
	public static final int RECOG_SMALL = 2; // 图像太小
	public static final int RECOG_BLUR = 3; // 图像模糊
	public static final int RECOG_LANGUAGE = 4; // 识别语言错误
	public static final int RECOG_IMEI_ERROR = 5;// IMEI号授权问题
	public static final int RECOG_FAIL_CDMA = 6; // CDMA限制
	public static final int RECOG_TIME_OUT = 7; // 引擎时间过期
	public static final int RECOG_LICENSE_ERROR = 8;// 授权问题
	public static final int RECOG_ENGINE_INIT_ERROR = 9;// 引擎初始化失败
	public static final int RECOG_COPY = 10; // 图像模糊
	public static final int RECOG_TYPE = 11;
	public static final int RECOG_IMG_ENGINE_INIT_ERROR = 12;

	//
	private static final int MIN_WIDTH_LIMIT = 1024;
	private static final int MIN_HEIGHT_LIMIT = 720;

	//
	// BField ID
	private static final int BIDC_NON = 0; // Blank field
	private static final int BIDC_NAME = 1; // 姓名
	private static final int BIDC_CARDNO = 3; // 身份证号
	private static final int BIDC_SEX = 4; // 性别
	private static final int BIDC_FOLK = 11; // 民族
	private static final int BIDC_BIRTHDAY = 5; // 生日
	private static final int BIDC_ADDRESS = 6; // 地址
	private static final int BIDC_ISSUE_AUTHORITY = 7; // 签发单位
	private static final int BIDC_VALID_PERIOD = 9; // 有效期
	private static final int BIDC_MEMO = 99; // 备忘

	//
	// member variables
	protected long[] ppEngine = null;
	protected long[] ppImage = null;
	protected long[] ppField = null;
	protected long pEngine = 0;
	protected long pImage = 0;
	protected long pField = 0;
	protected NativeOcr mNativeOcr = null;
	protected boolean mBeCancel = false;

	private static boolean OPT_CANCEL = false; // 识别中取消
	private List<String> list = new ArrayList<String>();

	// ///////////////////////////////////////////////////////
	// Standard Constructor/Destructor

	public OcrEngine() {
		ppEngine = new long[1];
		ppImage = new long[1];
		ppField = new long[1];
		mNativeOcr = new NativeOcr();
	}

	@Override
	public void finalize() {
		ppEngine = null;
		ppImage = null;
		ppField = null;
		mNativeOcr = null;

		pEngine = 0;
		pImage = 0;
	}

	// //////////////////////////////////////////////////////////////////
	// Operations

	// private boolean startBCR(String cfgPath, String dataPath, int lanuage,
	// byte[] license) {
	// boolean result = false;
	// int ret = mNativeOcr.startBCR(ppEngine,
	// StringUtil.convertUnicodeToAscii(dataPath),
	// StringUtil.convertUnicodeToAscii(cfgPath), lanuage, license);
	// // pEngine = ppEngine[0]; // keep the engine pointer
	// // Log.d("engine", ppEngine.length+"<----engine--->>"+pEngine);
	// if (ret == 1) {
	// pEngine = ppEngine[0]; // keep the engine pointer
	// result = true;
	// }
	// return result;
	// }

	private int startOCR(String cfgPath, String dataPath, int lanuage, int type) {
		int ret = mNativeOcr.startOCR(ppEngine, StringUtil.convertUnicodeToAscii(dataPath),
				StringUtil.convertUnicodeToAscii(cfgPath), lanuage, type);
		Log.d("engine", ret + "<----engine--->>");
		if (ret == 1) {
			pEngine = ppEngine[0]; // keep the engine pointer
		}
		return ret;
	}

	private int startBCR(String cfgPath, String dataPath, int lanuage, byte[] license) {
		int ret = mNativeOcr.startBCR(ppEngine, StringUtil.convertUnicodeToAscii(dataPath),
				StringUtil.convertUnicodeToAscii(cfgPath), lanuage, license);
		YMLogUtil.logD("--startBCR-ret->" + ret);
		if (ret == 1) {
			pEngine = ppEngine[0]; // keep the engine pointer
		}
		return ret;
	}

	private void closeBCR() {
		if (ppEngine != null && mNativeOcr != null) {
			mNativeOcr.closeBCR(ppEngine);
			ppEngine[0] = 0;
			pEngine = 0;
		}
	}

	private boolean loadImageMem(long pBuffer, int width, int height, int component) {
		boolean result = false;

		if (pBuffer != 0) {
			pImage = mNativeOcr.loadImageMem(pEngine, pBuffer, width, height, component);
			if (pImage != 0) {
				ppImage[0] = pImage;
				result = true;
			}
		}

		return result;
	}

	private boolean doImageBCR(String headPath, String imgPath) {
		boolean result = false;
		mBeCancel = false;
		// mNativeOcr.setoption(pEngine, StringUtil.convertToUnicode("-fmt"),
		// null);

		// byte[] buffer = null;
		// if (headPath != null && headPath.length() > 0) {
		// StringBuffer s = new StringBuffer(headPath);
		// s.append('0');
		// try {
		// buffer = s.toString().getBytes("gbk");
		// } catch (UnsupportedEncodingException e) {
		// // TODO Auto-generated catch block
		// e.printStackTrace();
		// }
		// buffer[buffer.length - 1] = 0;
		// }

		byte[] bufferHeadPath = StringUtil.convertUnicodeToGbk(headPath);
		byte[] bufferimgPath = StringUtil.convertUnicodeToGbk(imgPath);

		int ret = mNativeOcr.doImageBCR(pEngine, pImage, ppField, bufferHeadPath, bufferimgPath);
		if (ret == 1) {
			pField = ppField[0];
			result = true;
		} else if (ret == 3) {
			mBeCancel = true;
		}
		return result;
	}

	private String doLineOCR() {
		String result = "";
		byte[] res = new byte[1024];
		int ret = mNativeOcr.doLineOCR(pEngine, pImage, ppField, res, res.length);
		Log.d("engine", "----getNum---ret--->>" + ret);
		if (ret == 1) {
			pField = ppField[0];
		}
		try {
			result = new String(res, "gbk");
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result.trim();
	}

	private boolean isCancel() {
		return mBeCancel;
	}

	private void freeImage() {
		if (mNativeOcr != null) {
			mNativeOcr.freeImage(pEngine, ppImage);
			ppImage[0] = 0;
			pImage = 0;
		}
	}

	private void freeBFields() {
		if (mNativeOcr != null) {
			mNativeOcr.freeBField(pEngine, ppField[0], 0);
			ppField[0] = 0;
			pField = 0;
		}
	}

	private boolean isBlurImage() {
		boolean result = false;
		if (mNativeOcr != null) {
			int ret = mNativeOcr.imageChecking(pEngine, pImage, 0);
			if (ret == 2) {
				result = true;
			}
		}
		return result;
	}

	private boolean isCopyId() {
		boolean result = false;
		if (mNativeOcr != null) {
			int ret = mNativeOcr.checkingCopyID(pImage);
			Log.d("tag", "<ret>" + ret);
			if (ret == 1) {
				result = true;
			}
		}
		return result;
	}

	private boolean isCardId() {
		boolean result = false;
		if (mNativeOcr != null) {
			int ret = mNativeOcr.GetCardType(pEngine, pImage);
			Log.d("tag", pEngine + "<ret>" + ret);
			if (ret < 16 || ret > 24) {
				result = true;
			}
		}
		return result;
	}

	private int getFieldId() {
		long field = pField;
		return mNativeOcr.getFieldId(field);
	}

	private String getFieldText(int keyLanguage) {
		long field = pField;
		final int textsize = 256;
		byte[] text = new byte[textsize];
		mNativeOcr.getFieldText(field, text, textsize);

		int ocrLang = OCR_LAN_CHINESE_SIMPLE;
		if (keyLanguage == OCR_CODE_GB2B5 && ocrLang == OCR_LAN_CHINESE_SIMPLE) {
			mNativeOcr.codeConvert(pEngine, text, keyLanguage);
			return StringUtil.convertBig5ToUnicode(text);
		} else if (ocrLang == OCR_LAN_EUROPEAN) {
			return StringUtil.convertAscIIToUnicode(text);
		} else {
			return StringUtil.convertGbkToUnicode(text);
		}
	}

	/**
	 * 获取字段区域坐标
	 * 
	 * @return
	 */
	private Rect getFieldRect() {
		Rect rect = new Rect();
		long field = pField;
		int[] point = new int[4];
		mNativeOcr.getFieldRect(field, point);
		rect.left = point[0];
		rect.top = point[1];
		rect.right = point[2];
		rect.bottom = point[3];
		return rect;
	}

	private void getNextField() {
		if (!isFieldEnd()) {
			pField = mNativeOcr.getNextField(pField);
		}
	}

	private boolean isFieldEnd() {
		if (pField == 0) {
			return true;
		}
		return false;
	}

	private void setProgressFunc(boolean install) {
		if (pEngine != 0 && mNativeOcr != null) {
			mNativeOcr.setProgressFunc(pEngine, install);
		}
	}

	/**
	 * convert the result of recognization into IDCard format.
	 * 
	 * @param idCard
	 *            [out] get the result of recognization
	 * 
	 * @return true -- success false -- the idCard is null
	 */
	private boolean fields2Object(IDCard idCard, int keyLanguage) {
		if (idCard != null) {
			while (!isFieldEnd()) {
				switch (getFieldId()) {
				case BIDC_NAME:
					idCard.setName(getFieldText(keyLanguage));
					break;
				case BIDC_CARDNO:
					// idCard.setCardNo(getFieldText(keyLanguage));
					idCard.setCardNo(getFieldText(keyLanguage).replace("(wrong number)", ""));
					break;
				case BIDC_SEX:
					idCard.setSex(getFieldText(keyLanguage));
					break;
				case BIDC_FOLK:
					idCard.setEthnicity(getFieldText(keyLanguage));
					break;
				case BIDC_BIRTHDAY:
					idCard.setBirth(getFieldText(keyLanguage));
					break;
				case BIDC_ADDRESS:
					idCard.setAddress(getFieldText(keyLanguage));
					break;
				case BIDC_ISSUE_AUTHORITY:
					idCard.setAuthority(getFieldText(keyLanguage));
					break;
				case BIDC_VALID_PERIOD:
					idCard.setPeriod(getFieldText(keyLanguage));
					break;
				case BIDC_MEMO:
					idCard.setMemo(getFieldText(keyLanguage));
					break;
				}
				getNextField();
			}

			return true;
		}
		return false;
	}

	private boolean getNum(IDCard idCard, int keyLanguage) {
		if (idCard != null) {
			idCard.setCardNo(getFieldText(keyLanguage).replace("(wrong number)", ""));
			return true;
		}
		return false;
	}

	/**
	 * 取消识别
	 */
	public static void doCancel() {
		OPT_CANCEL = true;
	}

	/**
	 * 识别
	 * 
	 * @param context
	 * @param imagePath
	 *            图像路径
	 * @return 识别结果
	 * @throws IOException
	 */
	// public IDCard recognize(Context context, String imagePath,String
	// headPath) throws IOException {
	// return recognize(context, OCR_LAN_CHINESE_SIMPLE, imagePath,headPath);
	// }

	/**
	 * 识别
	 * 
	 * @param context
	 * @param imgBuffer
	 *            图像数据
	 * @return 识别结果
	 */
	public IDCard recognize(Context context, byte[] imgBuffer, String headPath) {
		return recognize(context, OCR_LAN_CHINESE_SIMPLE, imgBuffer, null, headPath, "");
	}

	/**
	 * 识别,正反两面
	 * 
	 * @param context
	 * @param idCardDataA
	 *            证件正面数据
	 * @param idCardDataB
	 *            证件背面数据
	 * @return 识别结果
	 */
	public IDCard recognize(Context context, byte[] idCardDataA, byte[] idCardDataB, String headPath) {
		return recognize(context, OCR_LAN_CHINESE_SIMPLE, idCardDataA, idCardDataB, headPath, "");
	}

	/**
	 * 识别,正反两面
	 * 
	 * @param context
	 * @param idCardDataA
	 *            证件正面数据
	 * @param idCardDataB
	 *            证件背面数据
	 * @return 识别结果
	 */
	public IDCard recognize(Context context, byte[] idCardDataA, byte[] idCardDataB, String headPath, String imgPath) {
		return recognize(context, OCR_LAN_CHINESE_SIMPLE, idCardDataA, idCardDataB, headPath, imgPath);
	}

	public IDCard recognize(Context context, String imagePath, String cutImagePath, String headPath) {
		int ocrLanguage = OCR_LAN_CHINESE_SIMPLE;

		IDCard idCard = new IDCard();

		// ocrLanguage = OCR_LAN_ENGLISH;//上海公安局 只证号读取版本 语言需设置为英文
		if (checkImei) {
			if (telephonyManager == null) {
				telephonyManager = (TelephonyManager) context.getSystemService(context.TELEPHONY_SERVICE);
			}
			String device = telephonyManager.getDeviceId().trim();
			Log.d("imei", "----imei------->>" + device);
			boolean isExit = showImei(device);
			if (!isExit) {
				idCard.setRecogStatus(RECOG_IMEI_ERROR);
				return idCard;
			}
		}

		if (checkCDMA) {
			boolean isCDMA = checkCDMA(context);
			if (!isCDMA) {
				idCard.setRecogStatus(OcrEngine.RECOG_FAIL_CDMA);
				return idCard;
			}
		}

		if (checkLicense) {
			try {
				List<String> list = getLicense();
				LicenseManager licenseManager = new LicenseManager(context, list.get(0), list.get(1));
				licenseManager.sendLicense();
			} catch (Exception e) {
				// TODO: handle exception
			}
		}

		int keyLanguage = OCR_CODE_GB;
		boolean blurDetection = false;
		if (ocrLanguage == OCR_LAN_CHINESE_TRADITIONAL) {
			keyLanguage = OCR_CODE_GB2B5;
		}

		OPT_CANCEL = false;

		byte[] license = new byte[256];
		AssetManager assetManager = context.getAssets();
		try {
			InputStream is = assetManager.open("license.info");
			is.read(license);
			is.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
		}
		// recognizatione
		switch (startBCR("", "", ocrLanguage, license)) {
		case 1:// 成功
			if (imagePath != null) {
				idCard = recognizing(imagePath, blurDetection, keyLanguage, headPath, cutImagePath);
			}
			closeBCR();
			break;
		case 100:// 过期
			idCard.setRecogStatus(RECOG_TIME_OUT);
			break;// 渠道号
		case 200:
			idCard.setRecogStatus(RECOG_LICENSE_ERROR);
			break;

		default:
			idCard.setRecogStatus(RECOG_ENGINE_INIT_ERROR);
			break;
		}
		return idCard;

	}

	public IDCard getCardNum(byte[] idCardDataA) {
		IDCard idCard = new IDCard();
		switch (startOCR("", "", OCR_LAN_ENGLISH, 2)) {
		case 1:// 成功
			if (idCardDataA != null) {
				idCard = recognizing(idCardDataA, OCR_CODE_GB);
			}
			// close OCR engine
			closeBCR();
			break;
		case 100:// 过期
			idCard.setRecogStatus(RECOG_TIME_OUT);
			break;// 渠道号
		case 200:
			idCard.setRecogStatus(RECOG_LICENSE_ERROR);
			break;

		default:
			idCard.setRecogStatus(RECOG_ENGINE_INIT_ERROR);
			break;
		}
		return idCard;
	}

	/**
	 * 识别
	 * 
	 * @param context
	 * @param ocrLanguage
	 *            识别语言：默认为中文简体:2
	 * @param imagePath
	 *            图像路径
	 * @return 识别结果
	 * @throws IOException
	 */
	private IDCard recognize(Context context, int ocrLanguage, String imagePath, String headPath) throws IOException {
		File file = new File(imagePath);
		byte[] imgBuffer = FileUtil.getBytesFromFile(file);
		return recognize(context, ocrLanguage, imgBuffer, null, headPath, "");
	}

	private boolean checkImei = false;
	private boolean checkCDMA = false;
	private boolean checkLicense = false;
	private boolean checkCopy = false;
	private boolean checkType = true;

	/**
	 * 识别
	 * 
	 * @param context
	 * @param ocrLanguage
	 *            识别语言
	 * @param idCardDataA
	 *            证件正面数据
	 * @param idCardDataB
	 *            证件背面数据
	 * @return 识别结果
	 */
	private IDCard recognize(Context context, int ocrLanguage, byte[] idCardDataA, byte[] idCardDataB, String headPath,
			String imgPath) {
		IDCard idCard = new IDCard();

		// ocrLanguage = OCR_LAN_ENGLISH;//上海公安局 只证号读取版本 语言需设置为英文
		if (checkImei) {
			if (telephonyManager == null) {
				telephonyManager = (TelephonyManager) context.getSystemService(context.TELEPHONY_SERVICE);
			}
			String device = telephonyManager.getDeviceId().trim();
			Log.d("imei", "----imei------->>" + device);
			boolean isExit = showImei(device);
			if (!isExit) {
				idCard.setRecogStatus(RECOG_IMEI_ERROR);
				return idCard;
			}
		}

		if (checkCDMA) {
			boolean isCDMA = checkCDMA(context);
			if (!isCDMA) {
				idCard.setRecogStatus(OcrEngine.RECOG_FAIL_CDMA);
				return idCard;
			}
		}

		if (checkLicense) {
			try {
				List<String> list = getLicense();
				LicenseManager licenseManager = new LicenseManager(context, list.get(0), list.get(1));
				licenseManager.sendLicense();
			} catch (Exception e) {
				// TODO: handle exception
			}
		}

		int keyLanguage = OCR_CODE_GB;
		boolean blurDetection = false;
		if (ocrLanguage == OCR_LAN_CHINESE_TRADITIONAL) {
			keyLanguage = OCR_CODE_GB2B5;
		}

		OPT_CANCEL = false;

		byte[] license = new byte[256];
		AssetManager assetManager = context.getAssets();
		try {
			InputStream is = assetManager.open("license.info");
			is.read(license);
			is.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
		}
		// recognizatione
		YMLogUtil.logD("--startBCR-s->");
		switch (startBCR("", "", ocrLanguage, license)) {
		case 1:// 成功
			if (idCardDataA != null) {
				idCard = recognizing(idCardDataA, blurDetection, keyLanguage, headPath, imgPath);
			}
			if (idCardDataB != null) {
				IDCard idCardB = recognizing(idCardDataB, blurDetection, keyLanguage, "", "");
				idCard.setAuthority(idCardB.getAuthority());
				idCard.setPeriod(idCardB.getPeriod());
			}
			// close OCR engine
			closeBCR();
			break;
		case 100:// 过期
			idCard.setRecogStatus(RECOG_TIME_OUT);
			break;// 渠道号
		case 200:
			idCard.setRecogStatus(RECOG_LICENSE_ERROR);
			break;

		default:
			idCard.setRecogStatus(RECOG_ENGINE_INIT_ERROR);
			break;
		}
		// if (startBCR("", "", ocrLanguage, license)) {
		// Log.d("tag", "---getLastErr--startBCR-ok--->>>");
		// if (idCardDataA != null) {
		// idCard = recognizing(idCardDataA, blurDetection, keyLanguage);
		// }
		// if (idCardDataB != null) {
		// IDCard idCardB = recognizing(idCardDataB, blurDetection,
		// keyLanguage);
		// idCard.setAuthority(idCardB.getAuthority());
		// idCard.setPeriod(idCardB.getPeriod());
		// }
		// // close OCR engine
		// closeBCR();
		// }else{
		// int ret = getLastErr();
		// Log.d("tag", "---getLastErr------>>>"+ret);
		// idCard.setRecogStatus(ret);
		// }
		return idCard;
	}

	private boolean showImei(String id) {
		// list.add("356108049077010");
		list.add("860486020187290");
		list.add("860173019034341");// 测试机 联想K900 小米1
		int x = 2;
		if (x == 1) {// 上海公安局
			list.add("356380052217276");
			list.add("356380052217284");
			list.add("356394051387741");
			list.add("356394051387758");
			list.add("867591010180784");
			list.add("862059010072380");
			list.add("356380052316136");
			list.add("356380052366396");
			list.add("356380053266404");
			list.add("356394052367163");
			list.add("356394052367171");
			list.add("862059010051301");
			list.add("356394052361448");
			list.add("356394052361455");
			list.add("356394051320460");
			list.add("356394051320478");

			list.add("868772013432047");// 第一次新增一个IMEI

			// list.add("861111023414520");//第二次新增三个IMEI
			// list.add("869631010714519");
			// list.add("869631010656470");

			list.add("99000455204849");// 第三次新增三个IMEI 第二次错误提供 注释掉
			list.add("A00000381618D8");
			list.add("A00000381618D85");
			list.add("A0000038162F84");
			list.add("A0000038162F844");
		} else if (x == 2) {// 北京电信
			list.add("353945053659693");
			list.add("863020013522333");
		}

		for (Iterator iterator = list.iterator(); iterator.hasNext();) {
			String str = iterator.next().toString();
			if (id.equals(str)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * 判断是否为CDMA手机
	 */
	private TelephonyManager telephonyManager;

	private boolean checkCDMA(Context context) {
		if (telephonyManager == null) {
			telephonyManager = (TelephonyManager) context.getSystemService(context.TELEPHONY_SERVICE);
		}
		return telephonyManager.getPhoneType() == TelephonyManager.PHONE_TYPE_CDMA;
	}

	/**
	 * 识别处理
	 * 
	 * @param imgBuffer
	 * @param blurDetection
	 * @param keyLanguage
	 * @return
	 */
	private IDCard recognizing(byte[] imgBuffer, boolean blurDetection, int keyLanguage, String headPath, String imgPath) {
		IDCard idCard = new IDCard();
		// load image from local
		ImageEngine imageEngine = new ImageEngine();
		if (imageEngine.init(ImageEngine.IMG_COMPONENT_GRAY, 100)) {
			YMLogUtil.logD("--imageEngine.load-s->");
			if (imageEngine.load(imgBuffer)) {
				// check size of image
				int width = imageEngine.getWidth();
				int height = imageEngine.getHeight();
				int component = imageEngine.getComponent();
				// if (android.os.Build.MODEL.contains("SO-01")
				// || android.os.Build.MODEL.contains("X10")
				// || android.os.Build.MODEL.contains("E10")) {
				//
				// } else {
				// if (width < MIN_WIDTH_LIMIT || height < MIN_HEIGHT_LIMIT) {
				// imageEngine.finalize(); // free image
				// imageEngine = null;
				// closeBCR(); // close OCR engine
				// idCard.setRecogStatus(RECOG_SMALL);
				// return idCard;
				// }
				// }
				long pBuffer = imageEngine.getDataEx();
				boolean loadImageMem = loadImageMem(pBuffer, width, height, component);
				// 识别引擎载入图像后 即可释放图像引擎
				// free image
				imageEngine.finalize();
				imageEngine = null;

				if (loadImageMem) {
					// long bak = dupImageOnly(new Rect(0, 0, width, height));
					// install callback function
					// setProgressFunc(true);

					if (checkCopy) {
						if (isCopyId()) {
							freeImage(); // detach image
							idCard.setRecogStatus(RECOG_COPY);
							return idCard;
						}
					}
					if (checkType) {
						idCard.setIDCard(!isCardId());
						// if(isCardId()){
						// freeImage(); // detach image
						// idCard.setRecogStatus(RECOG_TYPE);
						// return idCard;
						// }
					}

					if (blurDetection) { // blur detection dependent on
						// setting
						if (isBlurImage() && !OPT_CANCEL) {
							freeImage(); // detach image
							idCard.setRecogStatus(RECOG_BLUR);
							return idCard;
						}
					}

					// OCR recognize the IDCard
					YMLogUtil.logD("--doImageBCR.start-s->");
					if (doImageBCR(headPath, imgPath)) {
						// long bak = dupImageOnly(new Rect(0, 0, width,
						// height));
						// if ("" != path && null != path) {
						// saveImg(bak, path);
						// }
						// if(headPath != null && headPath.length() > 0){
						// long bak = dupImageOnly(new Rect(0, 0, width,
						// height));
						// saveImg(getheadImg(bak,getheadImgRect()), headPath);
						// // getheadImg(headPath);
						// }
						YMLogUtil.logD("--fields2Object.START-s->");
						if (fields2Object(idCard, keyLanguage)) {
							idCard.setRecogStatus(RECOG_OK);
							YMLogUtil.logD("--fields2Object.ok->");
						} else {
							YMLogUtil.logD("--fields2Object.faild->");
						}
						// release all fields
						freeBFields();
					} else if (isCancel()) {
						idCard.setRecogStatus(RECOG_CANCEL);
						// release all fields
						freeBFields();
					} else {
						YMLogUtil.logD("--fields2Object.faild-end->");
					}
					// detach image
					freeImage();
				}
			}
		}
		return idCard;
	}

	/**
	 * 识别处理
	 * 
	 * @param imgBuffer
	 * @param blurDetection
	 * @param keyLanguage
	 * @return
	 */
	private IDCard recognizing(String imagePath, boolean blurDetection, int keyLanguage, String headPath,
			String cutImagePath) {
		IDCard idCard = new IDCard();
		// load image from local
		ImageEngine imageEngine = new ImageEngine();
		if (imageEngine.init(ImageEngine.IMG_COMPONENT_GRAY, 100)) {
			// String copy = Environment.getExternalStorageDirectory().getPath()
			// + "/ytemp";
			// if (!copyFile(imgPath, copy)) {
			// idCard.setRecogStatus(RECOG_IMG_ENGINE_INIT_ERROR);
			// return idCard;
			// }

			if (imageEngine.load(imagePath)) {
				// check size of image
				int width = imageEngine.getWidth();
				int height = imageEngine.getHeight();
				int component = imageEngine.getComponent();
				// if (android.os.Build.MODEL.contains("SO-01")
				// || android.os.Build.MODEL.contains("X10")
				// || android.os.Build.MODEL.contains("E10")) {
				//
				// } else {
				// if (width < MIN_WIDTH_LIMIT || height < MIN_HEIGHT_LIMIT) {
				// imageEngine.finalize(); // free image
				// imageEngine = null;
				// closeBCR(); // close OCR engine
				// idCard.setRecogStatus(RECOG_SMALL);
				// return idCard;
				// }
				// }
				long pBuffer = imageEngine.getDataEx();
				boolean loadImageMem = loadImageMem(pBuffer, width, height, component);
				// 识别引擎载入图像后 即可释放图像引擎
				// free image
				imageEngine.finalize();
				imageEngine = null;

				if (loadImageMem) {
					// long bak = dupImageOnly(new Rect(0, 0, width, height));
					// install callback function
					// setProgressFunc(true);

					if (checkCopy) {
						if (isCopyId()) {
							freeImage(); // detach image
							idCard.setRecogStatus(RECOG_COPY);
							return idCard;
						}
					}
					if (checkType) {
						idCard.setIDCard(!isCardId());
						// if(isCardId()){
						// freeImage(); // detach image
						// idCard.setRecogStatus(RECOG_TYPE);
						// return idCard;
						// }
					}

					if (blurDetection) { // blur detection dependent on
						// setting
						if (isBlurImage() && !OPT_CANCEL) {
							freeImage(); // detach image
							idCard.setRecogStatus(RECOG_BLUR);
							return idCard;
						}
					}

					// OCR recognize the IDCard
					Log.d("engine", "----doImageBCR---start-----ocr--->>");
					if (doImageBCR(headPath, cutImagePath)) {
						// if(headPath != null && headPath.length() > 0){
						// long bak = dupImageOnly(new Rect(0, 0, width,
						// height));
						// saveImg(getheadImg(bak,getheadImgRect()), headPath);
						// // getheadImg(headPath);
						// }

						Log.d("engine", "----fields2Object---START-----ocr--->>");
						if (fields2Object(idCard, keyLanguage)) {
							idCard.setRecogStatus(RECOG_OK);
							Log.d("engine", "----fields2Object---OK-----ocr--->>");
						} else {
							Log.d("engine", "----fields2Object---faild-----ocr--->>");
						}
						// release all fields
						freeBFields();
					} else if (isCancel()) {
						idCard.setRecogStatus(RECOG_CANCEL);
						// release all fields
						freeBFields();
					} else {
						Log.d("engine", "----doImageBCR--faild-end-----ocr--->>");
					}
					// detach image
					freeImage();
				}
			}
		}
		return idCard;
	}

	private boolean copyFile(String imgPath, String copy) {
		File file = new File(copy);
		if (file.exists()) {
			file.delete();
		}
		return FileUtil.copyFile(imgPath, copy);
	}

	private IDCard recognizing(byte[] imgBuffer, int keyLanguage) {
		IDCard idCard = new IDCard();
		// load image from local
		ImageEngine imageEngine = new ImageEngine();
		if (imageEngine.init(ImageEngine.IMG_COMPONENT_GRAY, 90)) {
			if (imageEngine.load(imgBuffer)) {
				// check size of image
				int width = imageEngine.getWidth();
				int height = imageEngine.getHeight();
				int component = imageEngine.getComponent();
				long pBuffer = imageEngine.getDataEx();
				boolean loadImageMem = loadImageMem(pBuffer, width, height, component);
				// 识别引擎载入图像后 即可释放图像引擎
				// free image
				imageEngine.finalize();
				imageEngine = null;

				if (loadImageMem) {
					Log.d("engine", "----doLineOCR---start-----ocr--->>");
					idCard.setCardNo(doLineOCR());
					if (idCard.getCardNo().equals("")) {
						Log.d("engine", "----getNum---faild-----ocr--->>");
					} else {
						idCard.setRecogStatus(RECOG_OK);
						Log.d("engine", "----getNum---ok-----ocr--->>" + idCard.getCardNo());
					}
					freeBFields();
					// if (doLineOCR()) {
					// Log.d("engine", "----getNum---START-----ocr--->>");
					// if (getNum(idCard, keyLanguage)) {
					// idCard.setRecogStatus(RECOG_OK);
					// Log.d("engine", "----getNum---OK-----ocr--->>");
					// }else{
					// Log.d("engine", "----getNum---faild-----ocr--->>");
					// }
					// // release all fields
					// freeBFields();
					// } else if (isCancel()) {
					// idCard.setRecogStatus(RECOG_CANCEL);
					// // release all fields
					// freeBFields();
					// } else{
					// Log.d("engine",
					// "----doImageBCR--faild-end-----ocr--->>");
					// }
					// detach image
					freeImage();
				}
			}
		}
		return idCard;
	}

	public void saveImg(long pImage, String filePath) {
		if (mNativeOcr != null) {
			StringBuffer s = new StringBuffer(filePath);
			s.append('0');
			byte[] buffer = null;
			try {
				buffer = s.toString().getBytes("gbk");
			} catch (UnsupportedEncodingException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			buffer[buffer.length - 1] = 0;

			mNativeOcr.SaveImage(pImage, buffer);
		}
	}

	public void getheadImg(String filePath) {
		if (mNativeOcr != null) {
			StringBuffer s = new StringBuffer(filePath);
			s.append('0');
			byte[] buffer = null;
			try {
				buffer = s.toString().getBytes("gbk");
			} catch (UnsupportedEncodingException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			buffer[buffer.length - 1] = 0;

			mNativeOcr.getheadImg(pImage, pField, buffer);
		}
	}

	public long dupImageOnly(Rect rect) {
		if (rect == null) {
			return mNativeOcr.DupImage(pImage, new int[4]);
		}
		long result = -1;
		if (mNativeOcr != null) {
			int r[] = new int[4];
			r[0] = rect.left;
			r[1] = rect.top;
			r[2] = rect.right;
			r[3] = rect.bottom;
			result = mNativeOcr.DupImage(pImage, r);
		}
		return result;
	}

	public int[] getheadImgRect() {
		int r[] = new int[4];
		if (mNativeOcr != null) {
			mNativeOcr.getheadImgRect(pField, r);
		}
		return r;
	}

	public long getheadImg(long pImage, int[] rect) {
		long result = -1;
		if (mNativeOcr != null) {
			result = mNativeOcr.DupImage(pImage, rect);
		}
		return result;
	}

	private int getLastErr() {
		return mNativeOcr.getLastErr(pEngine, "".getBytes(), 0);
	}

	public List<String> getLicense() {
		String result = "";
		byte[] b = new byte[1024];
		int key = mNativeOcr.LicenseStr(b);
		try {
			result = new String(b, "gbk");
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			result = e.toString();
			e.printStackTrace();
		}
		List<String> list = new ArrayList<String>();
		list.add(result);
		list.add(String.valueOf(key));
		return list;
	}

	// public String getKey(){
	// return "18000";
	// }
}