package com.yunmai.android.other;

import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.ImageFormat;
import android.graphics.PixelFormat;
import android.hardware.Camera;
import android.hardware.Camera.AutoFocusCallback;
import android.hardware.Camera.Parameters;
import android.hardware.Camera.PictureCallback;
import android.hardware.Camera.ShutterCallback;
import android.hardware.Camera.Size;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.SurfaceHolder;

public class CameraManager implements AutoFocusCallback {
	
	public static final int mWidth = 1600;
	public static final int mHeight = 1200;
	
//	private Context mContext;
	private Camera mCamera;
	private Handler mHandler;
	private int takeType = 1;

	public CameraManager(Context context, Handler handler) {
		// TODO Auto-generated constructor stub
//		this.mContext = context;
		this.mHandler = handler;
	}

	public void openCamera(SurfaceHolder holder) throws RuntimeException, IOException {
		// TODO Auto-generated method stub
		if (mCamera == null) {
			mCamera = Camera.open();
			mCamera.setPreviewDisplay(holder);
//			setPictureSize();
			setPicSize();
		}
	}

	public void initDisplay() {
		// TODO Auto-generated method stub
		if (mCamera != null) {
			mCamera.startPreview();
		}
	}

	public void closeCamera() {
		// TODO Auto-generated method stub
		if (mCamera != null) {
			mCamera.stopPreview();
			mCamera.release();
			mCamera = null;
		}
	}

	public void requestFocuse() {
		// TODO Auto-generated method stub
		if (mCamera != null) {
			mCamera.autoFocus(this);
		}
	}
	
	/**
	 * 自动对焦
	 */
	public void autoFouce(){
		if (mCamera != null) {
			mCamera.autoFocus(autoFocusCallback);
		}
	}
	
	private AutoFocusCallback autoFocusCallback = new AutoFocusCallback() {
		
		@Override
		public void onAutoFocus(boolean success, Camera camera) {
			// TODO Auto-generated method stub
			
		}
	};
	
	
	public void takePicture(){
		if (mCamera != null) {
			try {
				Camera.Parameters parameters = mCamera.getParameters();
				parameters.setJpegQuality(100);
				mCamera.setParameters(parameters);
				mCamera.takePicture(shutterCallback, rawCallback, jpegCallback);
			} catch (RuntimeException e) {
			}
		}
	}
	
	/**
	 * 設置預覽大小
	 * @param width
	 * @param height
	 */
	public void setPreviewSize(int width, int height) {
		// TODO Auto-generated method stub
		if (mCamera != null) {
			Camera.Parameters parameters = mCamera.getParameters();
			List<Size> previewSize = parameters.getSupportedPreviewSizes();
			Collections.sort(previewSize, new SizeComparator());
			if (previewSize != null) {
				int size = previewSize.size();
				int picIndex = 0;
				for (int i = 0; i < size; i++) {
					if (previewSize.get(i).width == width) {
						picIndex = i;
						break;
					} else if (previewSize.get(i).width < width) {
						picIndex = i - 1;
						if (picIndex < 0) {
							picIndex = 0;
						}
						int diffW1 = previewSize.get(picIndex).width - width;
						int diffW2 = width - previewSize.get(i).width;
						if (diffW1 > diffW2) {
							picIndex = i;
						}
						break;
					}
				}
				if (Build.MODEL.startsWith("MI-ONE")) {
					if (Build.VERSION.INCREMENTAL.equals("2.10.12")) {
						parameters.setPreviewSize(640, 480);
					} else {
						parameters.setPreviewSize(1280, 720);
					}
				} else {
					parameters.setPreviewSize(previewSize.get(picIndex).width,
							previewSize.get(picIndex).height);
				}
			}
			mCamera.setParameters(parameters); 
		}
	}
	
	/**
	 * 設置照片大小
	 */
	private void setPictureSize() {
		// TODO Auto-generated method stub
		Camera.Parameters parameters = mCamera.getParameters();
		parameters.setPictureFormat(ImageFormat.JPEG);
		List<Size> pictureSize = parameters.getSupportedPictureSizes();
		Collections.sort(pictureSize, new SizeComparator());
		if (pictureSize != null) {
			int size = pictureSize.size();
			int picIndex = 0;
			for (int i = 0; i < size; i++) {
				if (pictureSize.get(i).width == mWidth) {
					picIndex = i;
					break;
				} else if (pictureSize.get(i).width < mWidth) {
					picIndex = i - 1;
					if (picIndex < 0) {
						picIndex = 0;
					}
					int diffW1 = pictureSize.get(picIndex).width - mWidth;
					int diffW2 = mWidth - pictureSize.get(i).width;
					if (diffW1 > diffW2 && pictureSize.get(i).width > 1280) {
						picIndex = i;
					}
					break;
				}
			}
			if (Build.MODEL.startsWith("HTC 606w") || Build.MODEL.startsWith("HTC 608t")) {
				parameters.setPictureSize(1600, 1200);
			} else {
				parameters.setPictureSize(pictureSize.get(picIndex).width,pictureSize.get(picIndex).height);
			}
		}
		mCamera.setParameters(parameters); 
	}
	
	
	
	public void setPicSize(){
		if(mCamera != null){
			Camera.Parameters parameters = mCamera.getParameters();
			parameters.setPictureFormat(PixelFormat.JPEG);
			List<Size> pictureSize = parameters.getSupportedPictureSizes();
			
			int x = 0;
			int tempSize = 2000;
			for (int i = 0; i < pictureSize.size(); i++) {
				if(pictureSize.get(i).width >= mWidth && pictureSize.get(i).width <= tempSize){
					tempSize = pictureSize.get(i).width;
//					Log.d(TAG, x+"<---x------->"+tempSize);
					x = i;
				}
			}
			parameters.setPictureSize(pictureSize.get(x).width,
					pictureSize.get(x).height);
			parameters.setJpegQuality(100);
			mCamera.setParameters(parameters);
			Log.d("tag", pictureSize.get(x).width+"<--w-pictureSize---h---->"+pictureSize.get(x).height);
			
			
		}
	}
	
	
	
	/**
	 * 设置闪关灯模式
	 * @param mode
	 */
	public void setCameraFlashMode(String mode) {
		// TODO Auto-generated method stub
		Camera.Parameters parameters = mCamera.getParameters(); 
		parameters.setFlashMode(mode); 
		mCamera.setParameters(parameters); 
	}
	
	/**
	 * 是否支持该模式闪关
	 * @param mode	闪光模式
	 * @return
	 */
	public boolean isSupportFlash(String mode) {
		// TODO Auto-generated method stub
		List<String> modes = mCamera.getParameters().getSupportedFlashModes();
		if (mCamera == null || modes == null || !modes.contains(mode)) {
			return false;
		}
		return true;
	}
	
	
	/**
	 * 判断/(设置)是否支持 自动对焦功能 
	 * @return
	 */
	public boolean isSupportAutoFocus(){
		List<String> list = getSupportedFocusModes();
		if(list == null) return false;
		for (String string : list) {
			if(Parameters.FOCUS_MODE_AUTO.equals(string)){
				return true;
			}
		}
		return false;
	}
	
	/**
	 * 判断/(设置)是否支持 对焦功能 
	 * @return
	 */
	public boolean isSupportFocus(String mode){
		List<String> list = getSupportedFocusModes();
		if(list == null) return false;
		for (String string : list) {
			if(mode.equals(string)){
				return true;
			}
		}
		return false;
	}
	
	/**
	 * 相机所能支持的聚焦模式
	 */
	private List<String> getSupportedFocusModes(){
		List<String> list = null;
		if(mCamera != null){
			Camera.Parameters parameters = mCamera.getParameters();
			list = parameters.getSupportedFocusModes();
			for (String string : list) {
				Log.d("path", "------SupportedFocusModes----------->>"+string);
			}
		}
		return list;
	}
	
	/**
	 * 设置选项，获取证件正面
	 */
	public void setTakeIdcardA() {
		this.takeType = 1;
	}
	
	/**
	 * 设置选项，获取证件反面
	 */
	public void setTakeIdcardB() {
		this.takeType = 2;
	}
	
	/**
	 * 获取默认闪关灯模式
	 * @return
	 */
	public String getDefaultFlashMode() {
		// TODO Auto-generated method stub
		if (mCamera.getParameters().getSupportedFlashModes() != null) {
			return mCamera.getParameters().getSupportedFlashModes().get(0);
		}
		return Parameters.FLASH_MODE_OFF;
	}

	@Override
	public void onAutoFocus(boolean success, Camera camera) {
		// TODO Auto-generated method stub
		takePicture(success);
	}
	
	private void takePicture(boolean captureOnly) throws RuntimeException {
		if (mCamera != null) {
			try {
				Camera.Parameters parameters = mCamera.getParameters();
				parameters.setJpegQuality(100);
				mCamera.setParameters(parameters);
				mCamera.takePicture(shutterCallback, rawCallback, jpegCallback);
			} catch (RuntimeException e) {
			}
		}
	}
	
	private ShutterCallback shutterCallback = new ShutterCallback() {
		
		public void onShutter() {
			/* 按下快门瞬间会调用这里的程序 */
		}
		
	};

	private PictureCallback rawCallback = new PictureCallback() {
		
		public void onPictureTaken(byte[] _data, Camera _camera) {
			/* 要处理raw data */
		}
		
	};

	private PictureCallback jpegCallback = new PictureCallback() {
		
		public void onPictureTaken(byte[] _data, Camera _camera) {
			try {
				String imageName = FileUtil.newImageName();
				Bitmap bm = BitmapFactory.decodeByteArray(_data, 0, _data.length);
//				File imageFile = new File(App.getImagesDir(), imageName);
//		        BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(imageFile));
//		        bm.compress(Bitmap.CompressFormat.JPEG, 50, bos);
//		        bos.flush();
//		        bos.close();
//		        bm.recycle();
//		        Message msg = new Message();
//		        msg.what = 1;
//		        msg.obj = imageName;
//		        mHandler.sendMessage(msg);

		        ByteArrayOutputStream bos = new ByteArrayOutputStream();
		        bm.compress(Bitmap.CompressFormat.JPEG, 50, bos);
		        Message msg = new Message();
		        msg.what = takeType;
		        msg.obj = imageName;
		        Bundle bundle = new Bundle();
		        bundle.putByteArray("picData", bos.toByteArray());
		        msg.setData(bundle);
		        mHandler.sendMessage(msg);
		        bos.flush();
		        bos.close();
		        bm.recycle();
			} catch (NullPointerException e) {
				mHandler.sendEmptyMessage(0);
			} catch (FileNotFoundException e) {
				mHandler.sendEmptyMessage(0);
			} catch (IOException e) {
				mHandler.sendEmptyMessage(0);
			}
		}
	};
	
	/**
	 * 倒序
	 */
	public class SizeComparator implements Comparator<Size> {

		@Override
		public int compare(Size s1, Size s2) {
			return s2.width*s2.height - s1.width*s1.height;
		}
		
	}

}
