package com.yunmai.cc.idcard.controler;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import android.app.Activity;
import android.content.Context;
import android.graphics.PixelFormat;
import android.graphics.Rect;
import android.hardware.Camera;
import android.hardware.Camera.AutoFocusCallback;
import android.hardware.Camera.Parameters;
import android.hardware.Camera.PreviewCallback;
import android.hardware.Camera.Size;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.Display;
import android.view.SurfaceHolder;
import android.view.WindowManager;


/**
 * տ
 * 
 * @author fangcm 2012-08-03
 *
 */

public class CameraManager implements AutoFocusCallback{

	private final String TAG = "cc_camera";
	private int mWidth = 1400;
	private int mPreviewWidth = 1280;
	private Handler mHandler;
	private Camera mCamera;
	private Context mContext;
	private boolean first = true;
	
	public CameraManager(Context context, Handler handler){
		this.mHandler = handler;
		this.mContext = context;
	}
	
	
	/**
	 * 
	 * @param holder
	 * @throws IOException
	 */
	public void openCamera(SurfaceHolder holder) throws IOException{
		closeCamera();
		if(mCamera == null)
			mCamera = Camera.open();
			mCamera.setPreviewDisplay(holder);
			setWhiteBalanceAuto();
	}
	
	public boolean cameraOpened(){
		return mCamera != null;
	}
	
	public void openCamera(){
		if(mCamera == null){
			mCamera = Camera.open();
			try {
				mCamera.setOneShotPreviewCallback(previewCallback);
			} catch (Exception e) {
				
			}
		}
	}

	
	public void setPreviewDisplay(SurfaceHolder holder){
		if(mCamera != null){
			try {
				mCamera.setPreviewDisplay(holder);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * ʼԤ
	 */
	public void initDisplay(){
		if(mCamera != null){
			try {
				mCamera.startPreview();
			} catch (Exception e) {
				// TODO: handle exception
			}
		}
	}
	
	
	/**
	 * ͣԤ
	 */
	public void pauseDisplay(){
		if(mCamera != null){
			mCamera.stopPreview();
		}
	}
	
	
	/**
	 * ر
	 * ΪԴ ر
	 */
	public void closeCamera(){
		if(mCamera != null){
			mCamera.stopPreview();
			mCamera.release();
			mCamera = null;
		}
	}
	
	
	/**
	 * ģʽ
	 * @param model
	 */
	public void setCameraFlashModel(String model){
		if(mCamera != null){
			Camera.Parameters parameters = mCamera.getParameters();
			parameters.setFlashMode(model);
			try {
				mCamera.setParameters(parameters);
			} catch (Exception e) {
				// TODO: handle exception
			}
		}
	} 
	
	
	
	/**
	 * ж/()Ƿ֧ ԶԽ 
	 * @return
	 */
	public boolean isSupportAutoFocus(){
		List<String> list = getSupportedFocusModes();
		if(list == null) return false;
		for (String string : list) {
			if(Parameters.FOCUS_MODE_AUTO.equals(string)){
				setFocusMode(Parameters.FOCUS_MODE_AUTO);
				return true;
			}
		}
		return false;
	}
	
	
	
	/**
	 * ʱ۽  
	 */
	public void requestFoucs(){
		if(mCamera != null){
			mCamera.autoFocus(this);
		}
	}
	
	
	public void setFocusMode(String mode){
		if(mCamera != null){
			try {
				Parameters parameters = mCamera.getParameters();
				parameters.setFocusMode(mode);
				mCamera.setParameters(parameters);
			} catch (Exception e) {
				// TODO: handle exception
			}
		}
	}
	
	
	/**
	 * Ƶ(Ԥ)С(ֱ)
	 * @param size
	 * @return
	 */
	public void setPreviewSize(int width, int height){
		try {
			Camera.Parameters parameters = mCamera.getParameters();
			parameters.setPreviewSize(width, height);
			mCamera.setParameters(parameters);
		} catch (Exception e) {
			// TODO: handle exception
		}
	}
	
	/**
	 * ƬС 
	 */
	public void setPicSize(Size optimalSize){
		if(mCamera != null){
			Camera.Parameters parameters = mCamera.getParameters();
			parameters.setPictureFormat(PixelFormat.JPEG);
			List<Size> pictureSize = parameters.getSupportedPictureSizes();
			if(optimalSize != null){
				parameters.setPictureSize(optimalSize.width,optimalSize.height);
				parameters.setJpegQuality(100);
				mCamera.setParameters(parameters);
				Log.d(TAG, optimalSize.width+"<--w-optimalPicSize---h---->"+optimalSize.height);
				return;
			}
			
			int x = 0;
			int tempSize = 3000;
			for (int i = 0; i < pictureSize.size(); i++) {
				if(pictureSize.get(i).width >= mWidth && pictureSize.get(i).width <= tempSize){
					tempSize = pictureSize.get(i).width;
					x = i;
				}
			}
			parameters.setPictureSize(pictureSize.get(x).width,
					pictureSize.get(x).height);
			parameters.setJpegQuality(100);
			mCamera.setParameters(parameters);
			Log.d(TAG, pictureSize.get(x).width+"<--w-pictureSize---h---->"+pictureSize.get(x).height);
		}
	}
	
	
	
	private Size getPreviewSizes(){
//		setPicSize(null);
		Size pSize = getPictureSizeForPreviewSize();
		if(pSize == null){
			setPicSize(null);
		}else{
			return pSize;
		}
		Camera.Parameters parameters = mCamera.getParameters();
		List<Size> prviewSizes = parameters.getSupportedPreviewSizes();
//		List<Size> prviewSizes = parameters.getSupportedPictureSizes();
		if(prviewSizes != null){
//			for (Size size : prviewSizes) {
//				Log.d(TAG, size.width+"<---x------->"+size.height);
//			}
			WindowManager windowManager = (WindowManager) mContext.getSystemService(Context.WINDOW_SERVICE);
			Display display = windowManager.getDefaultDisplay();
			int w = display.getWidth();
			if(w > mPreviewWidth){
				mPreviewWidth = w;
//				Size size = setPictureSize((double) display.getWidth() / display.getHeight());
//			    setPicSize(size);
				return null;
			}
			
			int x = 0;
			int tempSize = 0;
			for (int i = 0; i < prviewSizes.size(); i++) {
				if(prviewSizes.get(i).width <= mPreviewWidth && prviewSizes.get(i).width >= tempSize){
					tempSize = prviewSizes.get(i).width;
					x = i;
				}
			}
//			Size size = setPictureSize((double) prviewSizes.get(x).width / prviewSizes.get(x).height);
//		    setPicSize(size);
			return prviewSizes.get(x);
		}
		return null;
	}
	
	
	private Size getPictureSizeForPreviewSize(){
		Camera.Parameters parameters = mCamera.getParameters();
		List<Size> prviewSizes = parameters.getSupportedPreviewSizes();
		List<Size> pictureSize = parameters.getSupportedPictureSizes();
		List<Size> tempMaxSizes = new ArrayList<Camera.Size>();
		List<Size> tempMinSizes = new ArrayList<Camera.Size>();
		
		//ƬС  С
		Size tempSize;
		for (int j = pictureSize.size(); j >= 0; j--) {
			for (int i = 0; i < j - 1; i++) {
				if(pictureSize.get(i).width > pictureSize.get(i + 1).width){
					tempSize = pictureSize.get(i + 1);
					pictureSize.set(i + 1, pictureSize.get(i));
					pictureSize.set(i,tempSize);
				}
			}
		}
		
		for (int i = 0; i < pictureSize.size(); i++) {
			if(pictureSize.get(i).width >= mWidth ){
				tempMaxSizes.add(pictureSize.get(i));
			}else{
				tempMinSizes.add(pictureSize.get(i));
			}
		}
		
		//ԤС  С
		for (int j = prviewSizes.size(); j >= 0; j--) {
			for (int i = 0; i < j - 1; i++) {
				if(prviewSizes.get(i).width > prviewSizes.get(i + 1).width){
					tempSize = prviewSizes.get(i + 1);
					prviewSizes.set(i + 1, prviewSizes.get(i));
					prviewSizes.set(i,tempSize);
				}
			}
		}
		
		
		
		/**
		 * ɸѡ  
		 * ԤֱСڵ mPreviewWidth 
		 * Ƭֱʴڵ mWidth
		 *  ߱һ 
		 */
		for (int i = prviewSizes.size() - 1; i >= 0 ; i -- ) {
			if(prviewSizes.get(i).width <= mPreviewWidth){
				for (int j = 0; j < tempMaxSizes.size(); j++) {
//					Log.d(TAG, tempMaxSizes.get(j).width + "<-----MAX-size----->" +tempMaxSizes.get(j).height);
					if((double)prviewSizes.get(i).width / prviewSizes.get(i).height 
							== (double)tempMaxSizes.get(j).width / tempMaxSizes.get(j).height){
						setPicSize(tempMaxSizes.get(j));
//						Log.d(TAG, tempMaxSizes.get(j).width +"<---max-PIC-SIZE-->" + tempMaxSizes.get(j).height);
//						Log.d(TAG, prviewSizes.get(i).width +"<--max--pre-SIZE-->" + prviewSizes.get(i).height);
						return prviewSizes.get(i);
					}
				}
			}
		}
		
		
		/**
		 * ɸѡ  
		 * ԤֱСڵ mPreviewWidth 
		 * ƬֱС mWidth
		 *  ߱һ 
		 */
		for (int i = prviewSizes.size() - 1; i >= 0 ; i -- ) {
			if(prviewSizes.get(i).width <= mPreviewWidth){
				for (int j = tempMinSizes.size() - 1; j >= 0; j--) {
//					Log.d(TAG, tempMinSizes.get(j).width + "<-----MAX-size----->" +tempMinSizes.get(j).height);
					if(prviewSizes.get(i).width / prviewSizes.get(i).height 
							== tempMinSizes.get(j).width / tempMinSizes.get(j).height){
						setPicSize(tempMinSizes.get(j));
//						LOG.D(TAG, TEMPMINSIZES.GET(J).WIDTH +"<---MIN-PIC-SIZE-->" + TEMPMINSIZES.GET(J).HEIGHT);
//						Log.d(TAG, prviewSizes.get(i).width +"<--pre--PIC-SIZE-->" + prviewSizes.get(i).height);
						return prviewSizes.get(i);
					}
				}
			}
		}
		
		
		
		
//		for (int i = 0; i < tempMaxSizes.size(); i++) {
//			Log.d(TAG, tempMaxSizes.get(i).width + "<-----MAX-size----->" +tempMaxSizes.get(i).height);
//		}
//		
//		for (int i = 0; i < tempMinSizes.size(); i++) {
//			Log.d(TAG, tempMinSizes.get(i).width + "<-----MIN-size----->" +tempMinSizes.get(i).height);
//		}
		return null;
	}
	
	/**
	 * ԤС
	 *  -- Ե
	 */
	public void setPreviewSize(){
		if(mCamera == null){
			return;
		}
		Size size = getPreviewSizes();
		Camera.Parameters parameters = mCamera.getParameters();
//		parameters.setPreviewSize(w, h);
		if(size != null){
			Log.d(TAG, size.width+"<-------setPreviewSize--------->"+size.height);
			parameters.setPreviewSize(size.width, size.height);
		}else{
			WindowManager windowManager = (WindowManager) mContext.getSystemService(Context.WINDOW_SERVICE);
			Display display = windowManager.getDefaultDisplay();
			int w = display.getWidth();
			int h = display.getHeight();
			parameters.setPreviewSize(w, h);
		}
//		parameters.setPreviewSize(h, w);//
//		parameters.setPreviewSize(1600, 1200);
//		parameters.setPreviewSize(800, 480);
//		Log.d("path", w+"<----wh------------>"+h);
//		Log.d("path", size.width+"<----wh--szie---------->"+size.height);
		try {
			mCamera.setParameters(parameters);
		} catch (Exception e) {
			// TODO: handle exception
		}
		
	}
	
	
	
	
	/**
	 * ۽ 
	 */
	@Override
	public void onAutoFocus(boolean success, Camera camera) {
		// TODO Auto-generated method stub
		Log.d(TAG, "--onAutoFocus------>>>"+success);
		if(success){
//			takePicture();
		}else{
//			mHandler.sendEmptyMessage(UtilApp.DOC_CAMERA_AUTOFOCUS_FAILD);
		}
	}
	
	

	

	
	
	
	
	/**
	 * ƵʱȡԤͼ
	 */
	public void autoFocusAndPreviewCallback(){
		if(mCamera != null){
			try {
				first = false;
				mCamera.setOneShotPreviewCallback(previewCallback);
			} catch (Exception e) {
				// TODO: handle exception
				e.printStackTrace();
				mHandler.sendEmptyMessage(OcrConstant.START_AUTOFOCUS);
			}
		}
	}
	
	
	
	
	
	
	
	
	/**
	 * Խ
	 * ƵʼʱԽ
	 */
	public void autoFoucs(){
		try {
			mCamera.autoFocus(autoFocusCallback);
		} catch (Exception e) {
			// TODO: handle exception
		}
	}
	
	
	
	/**
	 * ԶԽ
	 */
	private Camera.AutoFocusCallback autoFocusCallback = new AutoFocusCallback() {
		
		@Override
		public void onAutoFocus(boolean success, Camera camera) {
			// TODO Auto-generated method stub
			Log.d(TAG, "---autoFoucs-------------->>"+success);
			if(success){
				try {
//					mCamera.setOneShotPreviewCallback(previewCallback);
				} catch (Exception e) {
					// TODO: handle exception
				}
			}
		}
	};
	
	
	
	
	
	/**
	 * ֵ֧ľ۽ģʽ
	 */
	private List<String> getSupportedFocusModes(){
		List<String> list = null;
		if(mCamera != null){
			Camera.Parameters parameters = mCamera.getParameters();
			list = parameters.getSupportedFocusModes();
			for (String string : list) {
				Log.d("path", "------SupportedFocusModes----------->>"+string);
			}
		}
		return list;
	}
	
	
	/**
	 * ðƽ
	 * 0 Զ
	 * @param model
	 */
	private void setWhiteBalanceAuto(){
		if(mCamera != null){
			Camera.Parameters parameters = mCamera.getParameters();
			parameters.setWhiteBalance(Parameters.WHITE_BALANCE_AUTO);
			try {
				mCamera.setParameters(parameters);
			} catch (Exception e) {
				// TODO: handle exception
				e.printStackTrace();
			}
		}
	}
	
	
	/**
	 * ȡƵԤͼ
	 */
	private Camera.PreviewCallback previewCallback = new PreviewCallback() {
		
		@Override
		public void onPreviewFrame(byte[] data, Camera camera) {
			// TODO Auto-generated method stub
			if(first) return;
			Message msg = mHandler.obtainMessage();
			msg.what = OcrConstant.TAKE_PREVIEW_DATA_OK;
			msg.obj = data;
			mHandler.sendMessage(msg);
		}
	};
	
	
	
	
	/**
	 * ȡӦͼƬΧ(getPreviewFrameForSDK_Min_8)
	 * @return
	 */
	public Rect getViewfinder(Rect finderRect){
		WindowManager windowManager = (WindowManager) mContext.getSystemService(Context.WINDOW_SERVICE);
		Display display = windowManager.getDefaultDisplay();
		float w = display.getWidth();
		float h = display.getHeight();
		int width =  mCamera.getParameters().getPreviewSize().width;
		int height = mCamera.getParameters().getPreviewSize().height;
		float xs = width/w;
		float ys = height/h;
		Rect rect = new Rect(finderRect);
		rect.left = (int) (finderRect.left * xs);
		rect.right = (int) (finderRect.right * xs);
		rect.top = (int) (finderRect.top * ys);
		rect.bottom = (int) (finderRect.bottom * ys);
		return rect;
	}
	
	
	public Rect getRectByPhoto(Rect finderRect){
		Rect previewRect = getViewfinder(finderRect);
		float w =  mCamera.getParameters().getPreviewSize().width;
		float h = mCamera.getParameters().getPreviewSize().height;
		float width =  mCamera.getParameters().getPictureSize().width;
		float height = mCamera.getParameters().getPictureSize().height;
		float xs = width/w ;
		float ys = height/h ;
		
		Rect rect = new Rect(previewRect);
		Log.d("ocr", rect.left+"<-----1--->"+rect.right +"<----->"+rect.top+"<------------>"+rect.bottom);
		rect.left = (int) (previewRect.left * xs);
		rect.right = (int) (previewRect.right * xs);
		rect.top = (int) (previewRect.top * ys);
		rect.bottom = (int) (previewRect.bottom * ys);
		Log.d("ocr", rect.left+"<-----2--->"+rect.right +"<----->"+rect.top+"<------------>"+rect.bottom);
		return rect;
	}
	
	
	
	
	
	/**
	 * ȡƵ(Ԥ)С(ֱʼ)
	 * @return
	 */
	public List<Size> getSupportedPreviewSizes(){
		if(mCamera != null ){
			return mCamera.getParameters().getSupportedPreviewSizes();
		}
		return null;
	}
	
	
	public Size getPicSize(){
		if(mCamera != null ){
			return mCamera.getParameters().getPictureSize();
		}
		return null;
	}
	
	public int getPreviewWidth(){
		if(mCamera != null ){
			return mCamera.getParameters().getPreviewSize().width;
		}
		return 0;
	}
	
	public int getPreviewHeight(){
		if(mCamera != null ){
			return mCamera.getParameters().getPreviewSize().height;
		}
		return 0;
	}
	
	public Size getOptimalPreviewSize(Activity currentActivity,
            List<Size> sizes, double targetRatio) {
        // Use a very small tolerance because we want an exact match.
        final double ASPECT_TOLERANCE = 0.001;
        if (sizes == null) return null;

        Size optimalSize = null;
        double minDiff = Double.MAX_VALUE;

        // Because of bugs of overlay and layout, we sometimes will try to
        // layout the viewfinder in the portrait orientation and thus get the
        // wrong size of mSurfaceView. When we change the preview size, the
        // new overlay will be created before the old one closed, which causes
        // an exception. For now, just get the screen size

        Display display = currentActivity.getWindowManager().getDefaultDisplay();
        int targetHeight = Math.min(display.getHeight(), display.getWidth());

        if (targetHeight <= 0) {
            // We don't know the size of SurfaceView, use screen height
            targetHeight = display.getHeight();
        }

        // Try to find an size match aspect ratio and size
        for (Size size : sizes) {
            double ratio = (double) size.width / size.height;
            if (Math.abs(ratio - targetRatio) > ASPECT_TOLERANCE) continue;
            if (Math.abs(size.height - targetHeight) < minDiff) {
                optimalSize = size;
                minDiff = Math.abs(size.height - targetHeight);
            }
        }

        // Cannot find the one match the aspect ratio. This should not happen.
        // Ignore the requirement.
        if (optimalSize == null) {
            Log.w(TAG, "No preview size match the aspect ratio");
            minDiff = Double.MAX_VALUE;
            for (Size size : sizes) {
                if (Math.abs(size.height - targetHeight) < minDiff) {
                    optimalSize = size;
                    minDiff = Math.abs(size.height - targetHeight);
                }
            }
        }
        return optimalSize;
    }
	
	
	
	public Size getOptimalPictureSize(List<Size> sizes, double targetRatio ,boolean isMax) {
		int tempMax = 100000000;
		int tempMin = 0;
		Size s = null;
        final double ASPECT_TOLERANCE = 0.001;
        if (sizes == null) return null;
        for (Size size : sizes) {
        	Log.d(TAG, size.width + "<-------getOptimalPictureSize---------->"+isMax);
            double ratio = (double) size.width / size.height;
            if (ratio == targetRatio){
            	if(isMax){
            		if(size.width * size.height < tempMax){
            			tempMax = size.width * size.height;
                		s = size;
                	}
            	}else{
            		if(size.width * size.height > tempMin){
                		tempMin = size.width * size.height;
                		s = size;
                	}
            	}
            }
//            if (Math.abs(ratio - targetRatio) > ASPECT_TOLERANCE || ratio == targetRatio){
//            	return size;
//            }
        }
        return s;
    }
	
	
	public boolean openFlashlight() {
		if (mCamera == null)
			mCamera = Camera.open();
		Camera.Parameters parameters = mCamera.getParameters();
		if(parameters.getSupportedFlashModes() == null ){
			return false;
		}
		if (parameters.getFlashMode().equalsIgnoreCase(Camera.Parameters.FLASH_MODE_TORCH))
			return false;
		
		parameters.setFlashMode(Camera.Parameters.FLASH_MODE_TORCH);
		try {
			mCamera.setParameters(parameters);
		} catch (Exception e) {
			// TODO: handle exception
		}
		if (!mCamera.getParameters().getFlashMode().equals(Camera.Parameters.FLASH_MODE_TORCH))
			return false;
		return true;
	}

	public boolean closeFlashlight() {
		if (mCamera == null)
			mCamera = Camera.open();
		if (mCamera.getParameters().getFlashMode().equalsIgnoreCase(Camera.Parameters.FLASH_MODE_OFF))
			return false;
		Camera.Parameters parameters = mCamera.getParameters();
		parameters.setFlashMode(Camera.Parameters.FLASH_MODE_OFF);
		try {
			mCamera.setParameters(parameters);
		} catch (Exception e) {
			// TODO: handle exception
		}
		if (!mCamera.getParameters().getFlashMode().equalsIgnoreCase(Camera.Parameters.FLASH_MODE_OFF))
			return false;
		return true;
	}

}
